%% EvaluateStudy: Create tabular overview of all file entries
function EvaluateStudy()

    clc;
    clear;
    close all;
    
    % Folder variables
    USE_TEMP = true;
    SHOW_TABLE = false;
    
    % Create global variabels
    global dictionary;
    global params;
    global num_params;
    global num_data;
    global methods;
    global groups;
    global group_id;
    global color_label;
    global color_value;
    global color_groups;
    global cur_field;
    global name_supp;
    global font;
    global create_figs;
    
    cur_field = 'DoubleGyreDeformed';
    
    dictionary = struct('CFTLE',1.0,'LFTLE',2.0,'RFTLE',3.0,'SineRidge',1.0,'DoubleGyreDeformed',2.0,'SpiralFocus',3.0,'DoubleGyreMirrored',4.0,'AN',1.0,'DC',2.0);
    color_label = [0.3 0.3 0.3];
    color_value = [0.5 0.5 0.5];
    color_groups = [0.4 0.5 0.65; 0.4 0.6 0; 0.9 0.2 0];
    group_id = 'METHOD';
    font = 'Calibri';
    name_supp = '';
    create_figs = true;
    
    methods = {'CFTLE';'LFTLE';'RFTLE'};
    fields = {'SineRidge';'DoubleGyreDeformed';'SpiralFocus';'DoubleGyreMirrored'};
    flags = {'AN';'DC'};
    data = [];
    
    % Define additional checks
    check_num = 2;
    checks = {'Ridges';'Measures'};
    
    % Start processing
    str = '[Loading Parameter Files...]'
    tic;
        
    if USE_TEMP
        % load temporary data
        [data,params,num_data,num_params] = loadTableData(cur_field);
        
        % get grouping
        gid = find(ismember(num_params, group_id)==1);
        groups = methods(cell2mat(data(:,gid)));
    end
    
    if ~USE_TEMP || size(data,1)<=1
      
        % Define base folders
        filetype = '*.am';
        cftle_path = ['J:\Automation\FTLEStudy\',cur_field,'\CFTLE\FTLE\'];
        rftle_path = ['J:\Automation\FTLEStudy\',cur_field,'\RFTLE\FTLE\'];
        lftle_path = ['J:\Automation\FTLEStudy\',cur_field,'\LFTLE\FTLE\'];

        % Get fileentries and parameters
        [cfent,cparams,cfnames] = getParameterFiles(cftle_path,filetype);
        [rfent,rparams,rfnames] = getParameterFiles(rftle_path,filetype);
        [lfent,lparams,lfnames] = getParameterFiles(lftle_path,filetype);

        % Combine all results
        fent = [cfent;rfent;lfent];
        params = unique([rparams;lparams;cparams]);
        params = [params;checks];
        fnames = [cfnames;rfnames;lfnames];

        file_num = size(fent,1)
        par_num = size(params,1)
        data = cell(file_num,par_num);
        measures = cell(size(data,1),1);
        groups = cell(size(data,1),1);

        toc;
        str = '[Processing File Entries...]'
        tic;
        
        % Check if files were found
        if (file_num>0 && par_num>0)

            % Create data entries
            for fc=1:file_num

                % get current file entry
                fentry = fent{fc};
                filename = fnames(fc);

                % check measures
                fentry = checkExistingMeasures(filename,fentry);

                % iterate all parameters
                for pc=1:par_num
                    if isfield(fentry,params{pc})
                        pval = fentry.(params{pc});
                        data(fc,pc) = {pval};
                    else
                        data(fc,pc) = {''};
                    end
                end

                % add measures as struct
                measures(fc) = {fentry.Errors};

                % add grouptype
                groups(fc) = methods(fentry.METHOD);
            end
            
            toc;
            str = '[Convert to numeric table...]'
            tic;

            % Get numeric data
            [num_data,num_params] = getNumericTable(data,measures,params);
            saveTableData(data,params,num_data,num_params,cur_field);
            
            toc;
            
            num_params
        end
    end
    
    toc;
    str = '[Plotting...]'
    tic;
    
    % Set Formats
    columnformat = cell(size(params,1),1);
    columnformat(1:end-check_num+1) = {'numeric'};
    columnformat((end-check_num+1):end) = {'logical'};

    % Create final table
    table_size = [10 10 1500 1000];
    fig_size = table_size*1.1;
    
    % Apply filtering
    fnd_re = filterNumericTable(num_data,num_params,{'RE_HAUSD_D1',0.000001,1000.0});
    fnd_gt = filterNumericTable(num_data,num_params,{'GT_L2_N0',0.000001,1000.0});
    fnd_all = filterNumericTable(fnd_re,num_params,{'GT_L2_N0',0.000001,1000.0});
    
    % Extract methods
    us = getUniqueSpaces(fnd_all,num_params,'METHOD');
    if ~isempty(us)
        facftle = us{1};
        farftle = us{2};
        falftle = us{3};
    end
    
    % Extract methods
    us = getUniqueSpaces(num_data,num_params,'METHOD');
    cftle = us{1};
    rftle = us{2};
    lftle = us{3};
    
    %% Evaluate results
      
    % Plot scatter matrices of base variables
    if false
        
        if false
            % Study Base Parameters 
            rbase_vars = {'TAU';'P0';'OR';'H';'FLAG';'HSAMP';'EPS1';'EPS2';'HEPS1';'HEPS2';'TIMING';'IR'};
            lbase_vars = {'TAU';'P0';'OR';'H';'FLAG';'HSAMP';'TIMING';'IR'};
            cbase_vars = {'TAU';'P0';'OR';'H';'FLAG';'TIMING';'IR'};
            
            %rbase_vars = {'TAU';'P0';'OR';'H';'HSAMP';'EPS1';'EPS2';'HEPS1';'HEPS2';'TIMING'};
            %lbase_vars = {'TAU';'P0';'OR';'H';'HSAMP';'TIMING'};
            %cbase_vars = {'TAU';'P0';'OR';'H';'TIMING'};
        else
            % Study Parameters & Measures
            cbase_vars = {'P0';'TAU';'H';'RE_MEAN_D1';'RE_MEAN_D12';'RE_HAUSD_D1';'RE_HAUSD_D12';'TIMING';'GT_L1_N0';'GT_L2_N0';'GT_LINF_N0'};
            lbase_vars = {'P0';'TAU';'H';'HSAMP';'RE_MEAN_D1';'RE_MEAN_D12';'RE_HAUSD_D1';'TIMING';'GT_L1_N0';'GT_L2_N0';'GT_LINF_N0'};
            rbase_vars = {'P0';'TAU';'H';'HSAMP';'EPS1';'HEPS2';'EPS2';'HEPS1';'RE_MEAN_D1';'RE_MEAN_D12';'RE_HAUSD_D1';'TIMING';'GT_L1_N0';'GT_L2_N0';'GT_LINF_N0'};
        end
        
        plot2DScatterMatrix(us{1},num_params,cbase_vars,'C-FTLE');
        %saveFigure('',[cur_field,'_CFTLE_ScatterMatrix'])
        
        plot2DScatterMatrix(us{2},num_params,rbase_vars,'R-FTLE');
        %saveFigure('',[cur_field,'_RFTLE_ScatterMatrix'])
        
        plot2DScatterMatrix(us{3},num_params,lbase_vars,'L-FTLE');
        %saveFigure('',[cur_field,'_LFTLE_ScatterMatrix'])
    end
    
    % Show table
    if SHOW_TABLE
        fig = figure('Position',table_size);
        table = uitable('Parent',fig,'Data',data,'ColumnName',params','ColumnFormat', columnformat','Position',table_size);
    end
    
    % Plot against common measures
    if false
        
        % Set base parameter
        curp = 'P0';
        
        % Apply basic filters
        frftle = filterNumericTable(rftle,num_params,{'TIMING',0.0001,10000;'GT_L2_N0',0.0001,0.5});
        flftle = filterNumericTable(lftle,num_params,{'TIMING',0.0001,10000;'GT_L2_N0',0.0001,0.5});
        fcftle = filterNumericTable(cftle,num_params,{'TIMING',0.0001,10000;'GT_L2_N0',0.0001,0.5});

        % Scatterplot values
        plot2DScatterplot(num_data,num_params,curp,'TIMING');
        plot2DScatterplot(num_data,num_params,curp,'RE_HAUSD_D1');
        plot2DScatterplot(num_data,num_params,curp,'GT_L2_N0');
        
        %upars = getUniqueParameters(num_data,num_params,{curp});
    end
    
    % Show unique surfaces
    if true
        
        %% DoubleGyre Deformed Field
        if strcmp(cur_field,'DoubleGyreDeformed')
            
            %Set intervals
            intv_h = [0.01,0.1];
            intv_tau = [4.0,8.0];
            hsamp_id = find(ismember(num_params, 'HSAMP')==1);
            
            % For cftle
            name_supp = 'C-FTLE';
            plot2DHeightfieldNG(cftle,num_params,'TAU','H','GT_L2_N0',intv_tau,intv_h);
            
            % For lftle
            hsamp_lus = getUniqueSpaces(lftle,num_params,{'HSAMP'});
            
            for uc=1:size(hsamp_lus,1) 
                hsamp_val = hsamp_lus{uc}(1,hsamp_id);
                name_supp = ['L-FTLE HSAMP=',num2str(hsamp_val)];
                plot2DHeightfieldNG(hsamp_lus{uc},num_params,'TAU','H','GT_L2_N0',intv_tau,intv_h); 
            end
            
            % For rftle
            hsamp_rus = getUniqueSpaces(rftle,num_params,{'HSAMP'});
            
            for uc=1:size(hsamp_rus,1) 
                hsamp_val = hsamp_lus{uc}(1,hsamp_id);
                name_supp = ['R-FTLE HSAMP=',num2str(hsamp_val)];
                plot2DHeightfieldNG(hsamp_rus{uc},num_params,'TAU','H','GT_L2_N0',intv_tau,intv_h); 
            end
            
        %% Sine Ridge Fields
        elseif strcmp(cur_field,'SineRidge')
            
            %target = 'GT_L2_N0';
            target = 'RE_HAUSD_D1';
            %target = 'TIMING';
            
            % Apply filtering
            fnd = filterNumericTable(num_data,num_params,{target,0.000001,100.0});
            
            % Extract methods
            fus = getUniqueSpaces(fnd,num_params,'METHOD');
            fcftle = fus{1};
            frftle = fus{2};
            flftle = fus{3};
            
            % C-FTLE H x TAU
            fcftle_us_flag = getUniqueSpaces(fcftle,num_params,{'FLAG'});       % split by flag
            cflag_an = fcftle_us_flag{1};   % analyitc
            cflag_dc = fcftle_us_flag{2};   % discrete
            
            fcftle_us_ir = getUniqueSpaces(cflag_an,num_params,{'IR'});         % split by input resolution
            fcftle_us_p0 = getUniqueSpaces(fcftle_us_ir{2},num_params,{'P0'});  % split by P0
            
            % Plot P0 sub surfaces
            if true
                p0num = size(fcftle_us_p0,1);
                %figure('Name','Combined Fields');
                %hold on;
                %create_figs = false;

                for pc=1:p0num, 
                    p0val = fcftle_us_p0{pc}(1,14);
                    name_supp = [cur_field,': P0=',num2str(p0val)];
                    plot2DHeightfieldNG(fcftle_us_p0{pc},num_params,'H','TAU',target); 
                end
            end
            
            % L-FTLE H x TAU
            flftle_us_flag = getUniqueSpaces(flftle,num_params,{'FLAG'});       % split by flag
            lflag_an = flftle_us_flag{1};   % analyitc
            lflag_dc = flftle_us_flag{2};   % discrete
            
            %flftle_us_ir = getUniqueSpaces(lflag_an,num_params,{'IR'});         % split by input resolution
            flftle_us_hsamp = getUniqueSpaces(lflag_an,num_params,{'HSAMP'});         % split by P0
            flftle_us_p0 = getUniqueSpaces(flftle_us_hsamp{1},num_params,{'P0'});         % split by P0
            
            if false
                p0num = size(flftle_us_p0,1);

                for pc=1:p0num, 
                    p0val = flftle_us_p0{pc}(1,14);
                    name_supp = [cur_field,': P0=',num2str(p0val)];
                    plot2DHeightfieldNG(flftle_us_p0{pc},num_params,'TAU','H',target); 
                end
            end
            
            %getUniqueParameters(flftle_us_ir{2},num_params)
            
            % R-FTLE H x TAU
            frftle_us_flag = getUniqueSpaces(frftle,num_params,{'FLAG'});       % split by flag
            rflag_an = frftle_us_flag{1};   % analyitc
            rflag_dc = frftle_us_flag{2};   % discrete
            
            frftle_us_ir = getUniqueSpaces(rflag_an,num_params,{'IR'});         % split by input resolution
            frftle_us_p0 = getUniqueSpaces(frftle_us_ir{2},num_params,{'P0'});  % split by P0
            
            %getUniqueParameters(frftle_us_ir{2},num_params)
            %getUniqueParameters(frftle_us_ir{1},num_params)
            
            % Show combined error plots
            if true
                plot2DHeightfield([cflag_an;lflag_an;rflag_an],num_params,'TAU','H',target,true,[1.2 3.0],[0.015 0.1]);
                plot2DHeightfield([cflag_an;lflag_dc;rflag_dc],num_params,'TAU','H',target,true,[1 3],[0.015 0.1]);
                
                %plot2DHeightfield([cflag_an;lflag_an;rflag_an],num_params,'TAU','H','TIMING',true,[1 3],[0.015 0.1]);
                %plot2DHeightfield([cflag_an;lflag_dc;rflag_dc],num_params,'TAU','H','TIMING',true,[1 3],[0.015 0.1]);
                
                plot2DHeightfield([cflag_an;lflag_an;rflag_an],num_params,'TAU','OR',target,true,[1.2 3.0],[0 300]);
                plot2DHeightfield([cflag_an;lflag_dc;rflag_dc],num_params,'TAU','OR',target,true,[1 3],[0 300]);
            end
            
            % Show Barplots
            if false 
                name_supp = [cur_field,' FLAG=AN '];
                plot2DBarplot([cflag_an;lflag_an;rflag_an],num_params,'METHOD','GT_L2_N0')
                plot2DBarplot([cflag_an;lflag_an;rflag_an],num_params,'METHOD','RE_HAUSD_D1')
                plot2DBarplot([cflag_an;lflag_an;rflag_an],num_params,'METHOD','TIMING')

                name_supp = [cur_field,' FLAG=DC '];
                plot2DBarplot([cflag_dc;lflag_an;rflag_dc],num_params,'METHOD','TIMING')
                plot2DBarplot([cflag_dc;lflag_an;rflag_dc],num_params,'METHOD','RE_HAUSD_D1')
                plot2DBarplot([cflag_dc;lflag_an;rflag_dc],num_params,'METHOD','GT_L2_N0')
            end
            
            % Show H x P0 Plots for specified TAU range
            if false
                tau_min = 1.5;
                tau_max = 2.2;

                name_supp = [cur_field,' TAU=[',num2str(tau_min),' .. ',num2str(tau_max),'] '];
                tfnd = filterNumericTable([fcftle;flftle;frftle],num_params,{'TAU',tau_min,tau_max});
                plot2DHeightfield(tfnd,num_params,'H','P0',target,true);

                % Min Max Mean Plots
                name_supp = [cur_field,' '];
                plot2DMinMaxMean(fnd,num_params,'H',target); 
                plot2DMinMaxMean(fnd,num_params,'TAU',target);
            end
            
        %% Spiral Focus Fields
        elseif strcmp(cur_field,'SpiralFocus')
            
            %target = 'GT_L2_N0';
            target = 'RE_HAUSD_D1';
            %target = 'TIMING';
            
            % Apply filtering
            fnd = filterNumericTable(num_data,num_params,{target,0.000001,100.0});
            fnd_tau10 = filterNumericTable(num_data,num_params,{'TAU',9.9,10.1});
            fnd_tau20 = filterNumericTable(num_data,num_params,{'TAU',19.9,20.1});
            
            % Extract methods
            fus = getUniqueSpaces(fnd,num_params,'METHOD');
            fcftle = fus{1};
            frftle = fus{2};
            flftle = fus{3};
            
            % Plot Min Max Mean Plots
            plot2DHeightfield(fnd,num_params,'TAU','H',target,true,[10 14.5],[0.035 0.1]);
            
            name_supp = [cur_field,' TAU=10 '];
            plot2DMinMaxMean(fnd_tau10,num_params,'H',target,[0.1 0.5]);
            legend({'R-FTLE Samples';'R-FTLE Mean';'R-FTLE Max';'R-FTLE Min';'R-FTLE';'L-FTLE Samples';'L-FTLE Mean';'L-FTLE Max';'L-FTLE Min';'L-FTLE';'C-FTLE Samples';'C-FTLE Mean';'C-FTLE Max';'C-FTLE Min';'C-FTLE'});
            
            name_supp = [cur_field,' TAU=20 '];
            plot2DMinMaxMean(fnd_tau20,num_params,'H',target,[0.1 0.5]);
            legend({'R-FTLE Samples';'R-FTLE Mean';'R-FTLE Max';'R-FTLE Min';'R-FTLE';'L-FTLE Samples';'L-FTLE Mean';'L-FTLE Max';'L-FTLE Min';'L-FTLE';'C-FTLE Samples';'C-FTLE Mean';'C-FTLE Max';'C-FTLE Min';'C-FTLE'});
        
            % Show Barplots
            if true 
                name_supp = [cur_field,' FLAG=AN '];
                %plot2DBarplot(fnd,num_params,'METHOD','GT_L2_N0');
                %plot2DBarplot(fnd,num_params,'METHOD','RE_HAUSD_D1');
                %plot2DBarplot(fnd,num_params,'METHOD','TIMING');
                
                name_supp = [cur_field,' TAU=10 FLAG=AN '];
                plot2DBarplot(fnd_tau10,num_params,'METHOD','RE_HAUSD_D1');
                plot2DBarplot(fnd_tau10,num_params,'METHOD','GT_L2_N0');
                
                name_supp = [cur_field,' TAU=20 FLAG=AN '];
                plot2DBarplot(fnd_tau20,num_params,'METHOD','RE_HAUSD_D1');
                plot2DBarplot(fnd_tau20,num_params,'METHOD','GT_L2_N0');
            end
        end
    end
    
    toc;
    
end

%% checkExistingMeasures: Check for existing measures
function [fentry]=checkExistingMeasures(filestruct,fentry)
    
    % Get Ridge path
    rf = strrep(filestruct.path,'\FTLE\','\Ridges\');
    rp = [rf,'Ridges_',strrep(filestruct.name,'.am','.mat')];
    fentry.Ridges = logical(exist(rp,'file'));
   
    % Get Measures
    mf = strrep(filestruct.path,'\FTLE\','\Measures\');
    mp = [mf,'Measures_',strrep(filestruct.name,'.am','.mat')];
    fentry.Measures = logical(exist(mp,'file'));
    fentry.Errors = struct();
    
    if (fentry.Measures) 
        fentry.Errors = load(mp); 
        
        % get all types
        fields = {'FTLE_MAX';'FTLE_MIN';...                     
        'RE_MEAN_D12';'RE_MEAN_D1';'RE_MEAN_D2';...
        'RE_HAUSD_D12';'RE_HAUSD_D1';'RE_HAUSD_D2';'GT_REMAIN0';...
        'GT_L2_N0';'GT_L1_N0';'GT_LINF_N0'};
    
        % check types
        for fc=1:size(fields,1)
            if ~isfield(fentry.Errors,fields(fc))
                fentry.Measures = false;
                break;
            end
        end
    end
    
    % Get timing
    tf = strrep(filestruct.name,'.am','.timing');
    tp = [filestruct.path,tf];
    timing = 0;
    
    if(exist(tp,'file'))
        fc = fileread(tp);
        timing = str2num(fc);
    end
    
    fentry.Errors.TIMING = timing;
    
end

%% getNumericTable: Convert cell table to numeric table
function [num_tab,nparams]=getNumericTable(data,measures,params)
    
    %Create empty table
    num_tab = zeros(size(data));
    num_tab = cellfun(@cellToNum,data,'UniformOutput',true);
    
    %Create empty table
    mparams = struct();
    mdata = zeros(size(measures));
    mid=1;
    
    % Iterate all file structs
    for fc=1:size(measures,1)
         fmstruct=measures{fc,1};
         fmnames = fieldnames(fmstruct);
         
         % Iterate all measure entries
         for nc=1:size(fmnames,1)
             fname = fmnames(nc);
             
             % Check if measure is already known
             if isfield(mparams,fname)
                 % Save measure entry in field
                 try
                    mdata(fc,mparams.(char(fname))) = fmstruct.(char(fname));
                 catch
                     disp('Something went wrong here...')
                 end
             else
                 % Add new entry and save measure entry
                 mdata(fc,mid) = fmstruct.(char(fname));
                 mparams.(char(fname))=mid;
                 mid=mid+1;
             end
         end
    end
    
    % Update parameter list
    nparams = [params;fieldnames(mparams)];
    
    % Update data
    num_tab = real([num_tab,mdata]);
    %size(mdata);
end

%% filterNumericTable: Filter given table using the respective data values
% filtervalues is a cell table containing {(string)Fieldname, (double)Minimum value, (double) Maximum Value}
function [ftable]=filterNumericTable(table, params, filtervalues)
    
    % Init filter parameters
    fpmin = 0;
    fpmax = 0;
    ftable = table;

    % check filter values
    if ~exist('filtervalues','var') || size(filtervalues,1)<1
        str = '[No filter values given.]'
        return;
    end
    
    % Iterate filter entries
    for fc=1:size(filtervalues,1)
        
        % get filter parameter name,id and value
        fpname = filtervalues{fc,1};
        fpid = find(ismember(params, fpname)==1);
        
        % select suitable values
        fpmin = filtervalues{fc,2};
        fpmax = filtervalues{fc,3};
        umask = logical((table(:,fpid)>=fpmin).*(table(:,fpid)<=fpmax));
        
        % apply filter data table if values left
        if sum(umask)>0
            ftable = table(umask,:);
        else
            disp(['[Filter ',fpname,' removed all values]']);
            ftable = zeros(1,size(table,2));
        end
    end
end

%% getUniqueSpaces: Determine unique data values in table
function [uspaces]=getUniqueSpaces(table, params, freevars)
    
    % Initiate values
    intable = table;
    uspaces = {};
    usidx=1;
    
    % Get variable names and ids
    if ~exist('freevars','var')
        %freevars={'TAU';'FLAG';'HSAMP';'EPS1';'EPS2';'HEPS1';'HEPS2'};
        freevars={'METHOD'};
        %fixedvars={'P0';'H'};
    end
    
    freevarids = find(ismember(params, freevars)==1);
    %fixedvarids = find(ismember(params, fixedvars)==1);
    
    % Iterate remaining table
    while size(intable,1)>2
        
        % Init arrays and parameters
        cur_vals = intable(1,freevarids);
        cuspace = zeros(1,size(intable,2));
        cuspace(1,:)=intable(1,:);
        mask = zeros(size(intable,1),1);
        mask(1)=1;
        cuid = 2;
        
        % Iterate remaining table values
        for ic=2:size(intable,1)
            % Check if row matches first entry
            if isequal(intable(ic,freevarids),cur_vals)
                cuspace(cuid,:) = intable(ic,:);
                mask(ic)=1;
                cuid=cuid+1;
            end
        end
        
        % Remove and save selected values
        intable(logical(mask),:)=[];
        uspaces(usidx)={cuspace};
        usidx=usidx+1;
    end
    
    % Transpose result
    uspaces = uspaces'; 
end
    
%% getUniqueParameters: Determine unique parameter combinations 
function [uparams]=getUniqueParameters(table, params, plotparams)
    
    % Initiate parameters
    pnum = size(params,1);
    uparams = cell(pnum,2);
    
    % Iterate parameter entries
    for tc=1:pnum
        % Get uniwue values
        uvalues = unique(table(:,tc));
        unum = size(uvalues,1);
        upname = params{tc};
        
        % Get values and number
        uhist = hist(table(:,tc),unum);
        uhist(2,:) = uvalues;
        
        %Save results
        uparams(tc,1) = {upname};
        uparams(tc,2) = {unum};
        uparams(tc,3) = {uhist};
    end
    
    % Plot selected parameters
    if exist('plotparams','var')
        
        % Iterate plot parameters
        for pc=1:size(plotparams,1)

            % Get Id and Name
            cpid = plotparams{pc,1};
            if ischar(cpid) cpid = find(ismember(params, cpid)==1); end
            cpname = char(params(cpid))
            
            % Create Figure
            ftitle = ['Parameter Propabilities ',cpname];
            fsizes = [0 0 800 800];
            setFigure(ftitle,fsizes);
            %hold on;

            % Plot occurences from histogram
            plot(uparams{cpid,3}(2,:),uparams{cpid,3}(1,:),'--s');
            uparams{cpid,3}

            % Set axes labels
            legend(cpname);
            setLabels(ftitle,'Parameter Value','Entry Count');
        end
    end
end

%% getMeanSurface: Compute unique mean surface for two parameters 
function [PX,PY,PZ]=getMeanSurface(table, params, pid1, pid2, tid)

    % Get Variable IDS
    if ischar(pid1) pid1 = find(ismember(params, pid1)==1); end
    if ischar(pid2) pid2 = find(ismember(params, pid2)==1); end
    if ischar(tid) tid = find(ismember(params, tid)==1); end
    %p1name = char(num_params(pid1));
    %p2name = char(num_params(pid2));
    %tname = char(num_params(tid));
    
    % Init arrays
    PX=0;
    PY=0;
    PZ=0;
    PNUM=1;
    uidx=1;
    %mean_table = zeros(1,size(table,2));
    
    % Iterate all table entries
    for tc=1:size(table,1)
        p1val = table(tc,pid1);
        p2val = table(tc,pid2);
        tval = table(tc,tid);
        FOUND = false;
        
        xid = find(PX(:,1)==p1val);
        
        % Check for entries
        if ~isempty(xid)
            
            % Get y index
            yid = find(PY(xid,1)==p2val);
            
            % Check tuple
            if ~isempty(yid)
                
                % Get x index
                zid = xid(yid,1);
                PZ(zid) = PZ(zid) + tval;
                PNUM(zid)=PNUM(zid)+1;
                %mean_table(zid)=mean_table(zid) + table(tc,:);
                
                FOUND = true;
            end
        end
            
        % No entries found, add new one
        if ~FOUND
            PX(uidx,1)=p1val;
            PY(uidx,1)=p2val;
            PZ(uidx,1)=tval;
            PNUM(uidx,1)=1;
            
            %mean_table(uidx)=table(tc,:);
            
            uidx = uidx+1;
        end
    end
    
    % Normalize values
    PZ = PZ./PNUM;
    
%     for tcc=1:size(table,2)
%         table(:,tcc)=table(:,tcc)./PNUM;
%     end
    
end

%% cellToNum: Convert cell entry to numeric value
function [value]=cellToNum(entry)
    
    global dictionary;

    % check if entry is character
    if ischar(entry)
        
        % convert to numeric value
        value = str2double(entry);
        
        % check entry type
        if strfind(entry,'x')
            value = str2num(entry(1:strfind(entry,'x')-1)); % Get resolutions
            return;
        elseif isfield(dictionary,entry);
            value = dictionary.(entry);                     % Get dictionary entries
            return;
        elseif isnan(value)
            value = 0;                                      % Catch undefined values
            return;
        end
    elseif isfloat(entry)
        %offset = 1e-10;
        offset = 0;
        value = double(entry)+offset;
    elseif isstruct(entry)
        
        fnames = fieldnames(entry);
        fname = '';
        value = zeros(size(fnames,1),1);
        
        for fc=1:size(fnames,1)
            fname = fnames(fc);
            value(fc,1)=entry.(char(fname));
        end
    else
        value = 0;                                          % no character entry
    end
end

%% plotParallelCoordinates: Plot parallel coordinates on all given measures
function plotParallelCoordinates(num_table,num_params,dtvars)
    
    global methods;
    global group_id;
    
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    %ugroup = unique(groups);
    
    % Filter data
    if(exist('dtvars','var'))
        num_data = num_table(1:end,dtvars);
        %groups = groups(1:end,1);
        lparams = num_params(dtvars,:)
    end
    
    % Plot figures
    size_fig = [0 0 1800 400];
    size_off = [0 0 400 0];
    
    num_data = checkTable(num_data);
    
    %% Parallel coordinates
    ftitle = 'Parallel Coordinates';
    setFigure(ftitle,size_fig);
    parallelcoords(num_data,'Labels',lparams,'Standardize','on','Group',groups,'LineWidth',1.5);
    setLabels(ftitle,'Measures','Measure Values');
    
    %% Andrews plot
    ftitle = 'Andrews Plot';
    setFigure(ftitle,size_fig+size_off);
    andrewsplot(num_data,'Standardize','on','Group',groups,'LineWidth',1.0,'Quantile',0.25);
    setLabels(ftitle,'Measures','Measure Values');
end

%% plot2DScatterplot: Plot 2D scatter plot function from base variable to target variable
function plot2DScatterplot(num_table,num_params,bid,tid)

    global methods;
    global group_id;
    global color_groups;
    global name_supp;
    
    % Get Variable IDS
    if ischar(bid) bid = find(ismember(num_params, bid)==1); end
    if ischar(tid) tid = find(ismember(num_params, tid)==1); end
    bname = char(num_params(bid));
    tname = char(num_params(tid));
    
    % Set Figure
    size_fig = [0 0 800 400];
    ftitle = ['2D Plot: ',name_supp,bname,' - ',tname];
    setFigure(ftitle,size_fig);
    hold on;
   
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
    
    %mean_fx = zeros(size(ugroup,1),1);
    %mean_ux = unique(num_table(:,gid));
    
    %for uc=1:size(ugroup,1)
    for uc=size(ugroup,1):-1:1
        % Filter data
        umask = strcmp(groups,ugroup(uc));
        ftable = num_table(umask,:);
        
        X = ftable(:,bid);
        FX = ftable(:,tid);

        % Plot data
        if ~isempty(X) && ~isempty(FX)
            plot(X,FX,'o','LineWidth',2.0,'MarkerSize',3.5,'Color',color_groups(uc,:));
        end
        
        % Compute mean
        %mean_fx(uc)=mean(FX);
    end
    
    % Set labels
    %legend(ugroup);
    legend(flipdim(ugroup,1));
    setLabels(ftitle,bname,tname);
    size(num_table);
    
end

%% plot2DBarplot: Plot 2D scatter plot function from base variable to target variable
function plot2DBarplot(num_table,num_params,bid,tid)

    global methods;
    global group_id;
    global color_groups;
    global name_supp;
    
    % Get Variable IDS
    if ischar(bid) bid = find(ismember(num_params, bid)==1); end
    bname = char(num_params(bid));
    
    if ischar(tid) tid = find(ismember(num_params, tid)==1); end
    tname = char(num_params(tid));
    
    % Set Figure
    size_fig = [0 0 500 800];
    %ftitle = ['2D Barplot: ',name_supp,bname,' - ',tname];
    ftitle = [name_supp,bname,' - ',tname];
    setFigure(ftitle,size_fig);
    hold on;
    colormap(color_groups);
   
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
    
    mean_fx = zeros(size(ugroup,1),1);
    mean_ux = unique(num_table(:,gid));
    
    for uc=1:size(ugroup,1)
        % Filter data
        umask = strcmp(groups,ugroup(uc));
        ftable = num_table(umask,:);
        
        % Get values
        X = ftable(:,bid);
        FX = ftable(:,tid);

        % Compute mean
        mean_fx(uc)=mean(FX);
    end
    
    % Plot bars
    %bar(mean_fx,'grouped');
    bar([mean_fx, zeros(size(mean_fx))]','grouped');
    set(gca,'XLim',[0.5 1.5]);
    
    % Set labels
    legend(ugroup);
    setLabels(ftitle,bname,tname);
    size(num_table);
    
end

%% plot2DScatterMatrix: Plot scatter plot matrix over all target parameters
% cparams is a cell array with target parameter string names
function plot2DScatterMatrix(num_table,num_params,cparams,nameadd)
    
    global color_groups;
    global font;

    if ~exist('nameadd','var') nameadd=''; end
    
    % Get Variable IDS
    %ngroups = cell2mat(cellfun(@(x)  find(ismember(methods,x)==1), groups, 'UniformOutput', false));
    cparam_ids = find(ismember(num_params, cparams)==1);
    cparams = num_params(cparam_ids,:)
    
    % Set Figure
    size_fig = [0 0 1200 1200];
    marker_type= '*';
    
    ftitle = ['Scatter Plot Matrix: ',nameadd];
    setFigure(ftitle,size_fig);
    hold on;
    %cla;
    
    % Plot data
    [H,AX,BigAx,P] = plotmatrix(num_table(:,cparam_ids),marker_type);
    %gplotmatrix(num_table(:,cparam_ids),ngroups);
    
    % Set text
    %text(gcf,1.0,1.0,{'RE_HAUSD_D1';'RE_HAUSD_D1'},'FontSize',14,'Color',[1 0 0]);
    axenum = size(AX,1);
    for axc=1:axenum 
        ann = strrep(cparams(axc),'_',' ');
        set(get(AX(axenum,axc),'XLabel'),'String',ann,'FontWeight','bold','FontName',font);
        set(get(AX(axc,1),'YLabel'),'String',ann,'FontWeight','bold','FontName',font);
        
        set(AX(axc,1),'FontSize',8,'FontWeight','bold','FontName',font);
        set(AX(axenum,axc),'FontSize',8,'FontWeight','light','FontName',font);
    end

    % Set labels
    xparams = cparams';
    xparams(2,:)={'                '};
    xchar = char([xparams{1:2,:}]);
    xchar = xchar(1:end-1);
    %xparams(3,:)={1:size(cparams,1)};
    
    cla;
    %setLabels(ftitle,xchar,xchar);
    setLabels(ftitle,'','');
end

%% plot2DBoxplot: Plot 2D function from base variable to target variable
function plot2DBoxplot(num_table,num_params,bid,tid)
    
    global methods;
    global group_id;
    global color_groups;
    
    % Get Variable IDS
    if ischar(bid) bid = find(ismember(num_params, bid)==1); end
    if ischar(tid) tid = find(ismember(num_params, tid)==1); end
    bname = char(num_params(bid));
    tname = char(num_params(tid));
    
    % Set Figure
    size_fig = [0 0 800 400];
    ftitle = ['2D Deviation Plot: ',bname,' - ',tname];
    setFigure(ftitle,size_fig);
    hold on;
   
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
       
    X = num_table(:,bid);
    FX = num_table(:,tid);

    % Plot data
    if ~isempty(X) && ~isempty(FX)
        %boxplot([X,FX],groups,'plotstyle','compact');
        %boxplot(X,groups,'plotstyle','compact');
        boxplot(FX,groups);
    end
    
    % Set labels
    legend(ugroup);
    setLabels(ftitle,bname,tname);
    size(num_table);
end

%% plot2DMinMaxMean: Plot two variables using a min, max, mean plot
function plot2DMinMaxMean(num_table,num_params,bid,tid,bint,tint)
 
    global methods;
    global group_id;
    global color_groups;
    global name_supp;
    
    % Get Variable IDS
    if ischar(bid) bid = find(ismember(num_params, bid)==1); end
    if ischar(tid) tid = find(ismember(num_params, tid)==1); end
    bname = char(num_params(bid));
    tname = char(num_params(tid));
    
    % Set Figure
    size_fig = [0 0 1200 400];
    ftitle = ['2D Average Plot: ',name_supp,bname,' - ',tname];
    setFigure(ftitle,size_fig);
    hold on;
    
    % Apply interval filters
    if exist('bint','var') num_table = filterNumericTable(num_table,num_params,{bname,bint(1),bint(2)}); end
    if exist('tint','var') num_table = filterNumericTable(num_table,num_params,{tname,tint(1),tint(2)}); end
    
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
    
    % Iterate groups
    %for uc=1:size(ugroup,1)
    for uc=size(ugroup,1):-1:1
        
        % Filter data
        umask = strcmp(groups,ugroup(uc));
        ftable = num_table(umask,:);
        
        X = ftable(:,bid);
        FX = ftable(:,tid);

        % plot values if defined
        if (~isempty(X) && ~isempty(FX)) 

            % Get unique x entries
            UX = unique(X);
            index = 1;
            unum = size(UX,1);
            
            uni_varye = cell(unum,1);
            var_ymin = zeros(unum,1);
            var_ymax = zeros(unum,1);
            var_ymean = zeros(unum,1);

            % Find all respective y entries
            for vuc=UX'
                %vuni = ftable(ftable(:,bid)==vuc,tid);
                vuni = FX(X==vuc,:);
                uni_varye{index,1} = vuni';
                index = index+1;
            end

            % get extrema and mean
            for euc=1:unum
                var_ymin(euc,1) = min(uni_varye{euc});
                var_ymax(euc,1) = max(uni_varye{euc});
                var_ymean(euc,1) = mean(uni_varye{euc});
            end

            % plot 3 graphs
            color = color_groups(uc,:);

            fi_x = [UX',fliplr(UX')];
            fi_y = [var_ymax',fliplr(var_ymin')];

            plot(X,FX,'o','MarkerEdgeColor',color,'MarkerFaceColor','none','MarkerSize',1);
            
            plot(UX,var_ymean,'.-','Color',color,'LineWidth',3.0);
            plot(UX,var_ymin,'.-','Color',color,'LineWidth',0.25);
            plot(UX,var_ymax,'.-','Color',color,'LineWidth',0.25);
            
            fill(fi_x,fi_y,color,'FaceAlpha',0.05,'EdgeColor',color,'EdgeAlpha',0.8);
        end
    end
    
    % Set labels
    %legend(sort([ugroup;ugroup]));
    legend({'R-FTLE Samples';'R-FTLE Mean';'R-FTLE Max';'R-FTLE Min';'R-FTLE';'L-FTLE Samples';'L-FTLE Mean';'L-FTLE Max';'L-FTLE Min';'L-FTLE';'C-FTLE Samples';'C-FTLE Mean';'C-FTLE Max';'C-FTLE Min';'C-FTLE'})
    %legend(flipdim(sort([ugroup;ugroup]),1));
    setLabels(ftitle,bname,tname);
    size(num_table);
end

%% plot2DBinningplot: Plot two variables while the first is binned to reduce scattering
function plot2DBinningplot(num_table,num_params,bid,tid,num_bins,bint,tint)
 
    global methods;
    global group_id;
    global color_groups;
    global name_supp;
    
    % Get Variable IDS
    if ischar(bid) bid = find(ismember(num_params, bid)==1); end
    if ischar(tid) tid = find(ismember(num_params, tid)==1); end
    bname = char(num_params(bid));
    tname = char(num_params(tid));
    
    % Set Figure
    size_fig = [0 0 500 500];
    ftitle = ['2D Average Plot: ',name_supp,bname,' - ',tname];
    setFigure(ftitle,size_fig);
    hold on;
    
    % Apply interval filters
    if exist('bint','var') num_table = filterNumericTable(num_table,num_params,{bname,bint(1),bint(2)}); end
    if exist('tint','var') num_table = filterNumericTable(num_table,num_params,{tname,tint(1),tint(2)}); end
    
    % Get group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
    
    % Iterate groups
    %for uc=1:size(ugroup,1)
    for uc=size(ugroup,1):-1:1
        
        % Filter data
        umask = strcmp(groups,ugroup(uc));
        ftable = num_table(umask,:);
        
        X = ftable(:,bid);
        FX = ftable(:,tid);
        
        % Renormalize binning variable
        X = (X-min(X))./(max(X)-min(X));
        FX = (FX-min(FX))./(max(FX)-min(FX));

        % plot values if defined
        if (~isempty(X) && ~isempty(FX)) 

            % Binning
            topEdge = 1; % define limits
            botEdge = 0; % define limits
            binEdges = linspace(botEdge, topEdge, num_bins+1);

            [h,whichBin] = histc(X, binEdges);

            binMean = zeros(num_bins,1);
            binMin = zeros(num_bins,1);
            binMax = zeros(num_bins,1);

            for i = 1:num_bins
                flagBinMembers = (whichBin == i);
                binMembers     = FX(flagBinMembers);

                if ~isempty(binMembers) 
                    binMean(i,1)    = mean(binMembers);
                    binMin(i,1)     = min(binMembers); 
                    binMax(i,1)     = max(binMembers);
                end
            end

            % plot 3 graphs
            
            %figure();
            color = color_groups(uc,:);
            hold on;

            %plot(binEdges(1:end-1),binMean,'LineWidth',3.0);
            %plot(binEdges(1:end-1),binMin,'LineWidth',0.5,'Color',[0 0.6 0]);
            %plot(binEdges(1:end-1),binMax,'LineWidth',0.5,'Color',[0.6 0 0]);

             fi_x = [binEdges',fliplr(binEdges')];
             fi_y = [binMax',fliplr(binMin')];
 
             
             sph = plot(X,FX,'o','MarkerEdgeColor',color,'MarkerFaceColor','none','MarkerSize',1);
             
             %bwh = plot(binEdges(1:end-1),binMean,'.-','Color',[1 1 1],'LineWidth',5.0);
             %bh = plot(binEdges(1:end-1),binMean,'.-','Color',color,'LineWidth',3.0);
             
             lsp = linspace(0,1,100);
             bsph = plot(lsp,interp1(binEdges(1:end-1),binMean,lsp,'pchip'),'.-','Color',color,'LineWidth',3.0);
             
             uistack(sph,'bottom');
             uistack(bsph,'top');
             
             %uistack(bh,'top');
             %plot(binEdges(1:end-1),binMin,'.-','Color',color,'LineWidth',0.25);
             %plot(binEdges(1:end-1),binMax,'.-','Color',color,'LineWidth',0.25);
             
             %fill(fi_x,fi_y,color,'FaceAlpha',0.05,'EdgeColor',color,'EdgeAlpha',0.8);
        end
    end
    
    % Set labels
    %legend(sort([ugroup;ugroup]));
    legend({'R-FTLE Samples';'L-FTLE Samples';'C-FTLE Samples';'R-FTLE Mean';'L-FTLE Mean';'C-FTLE Mean'})
    %legend(flipdim(sort([ugroup;ugroup]),1));
    setLabels(ftitle,bname,tname);
    size(num_table);
end


%% plot2DHeightfield: Plot function over two variables as 2D Heightfield for all methods
function plot2DHeightfield(num_table,num_params,v1id,v2id,tvid,GLOBAL_SCALE,intv1,intv2)

    global methods;
    global group_id;
    global color_groups;
    global name_supp;
    
    % Get Variable IDS
    if ischar(v1id) v1id = find(ismember(num_params, v1id)==1); end
    if ischar(v2id) v2id = find(ismember(num_params, v2id)==1); end
    if ischar(tvid) tvid = find(ismember(num_params, tvid)==1); end
    
    v1name = char(num_params(v1id));
    v2name = char(num_params(v2id));
    tname = char(num_params(tvid));
    
    % Set Figure
    size_fig = [0 0 1500 500];
    size_markers = 5.0;
    
    ftitle = ['2D Heightfield: ',name_supp,' ',v1name,',',v2name,' x ',tname];
    setFigure(ftitle,size_fig);
    %hold on;
    colormap jet;
    
    % Iterate group sets
    gid = find(ismember(num_params, group_id)==1);
    groups = methods(uint8(num_table(:,gid)));
    ugroup = unique(groups);
    unum = size(ugroup,1);
    
    % Rescale global min / max value
    if exist('GLOBAL_SCALE','var') && GLOBAL_SCALE
        APZ = checkTable(num_table(:,tvid));
        amin = min(APZ);
        amax = max(APZ);
    end
    
    % Iterate group data
    for uc=1:unum
        
        % Filter data for groups
        umask = strcmp(groups,ugroup(uc));
        ftable = num_table(umask,:);
        
        PX = ftable(:,v1id);
        PY = ftable(:,v2id);
        PZ = ftable(:,tvid);
        
        % Apply interval filters
        if exist('intv1','var')
             mpx = ~logical((PX>=intv1(1)).*(PX<=intv1(2)));
             PX(mpx)=[]; PY(mpx)=[]; PZ(mpx)=[];
        end

        if exist('intv2','var')
             mpy = ~logical((PY>=intv2(1)).*(PY<=intv2(2)));
             PX(mpy)=[]; PY(mpy)=[]; PZ(mpy)=[];
        end
        
        % Create subplot
        subplot(1,unum,uc);
        hold on;
        sptitle = [char(ugroup(uc)),': ',tname];
        colorbar('location','SouthOutside');
        
        % Remove undefined values
        PZ = checkTable(PZ);
        
        % Check for remaining values
        if (~isempty(PZ))

            % Delaunay triangulation of unique points
            if (false) 
                CX = [PX,PY];
                [~, I, ~] = unique(CX,'first','rows');
                I = sort(I);

                PX = PX(I,:);
                PY = PY(I,:);
                PZ = PZ(I,:);
            end
            
            % Rescale to remaining values
            if exist('GLOBAL_SCALE','var') && GLOBAL_SCALE
                caxis([amin amax]);
            else
                caxis([min(PZ) max(PZ)]);
            end

            try
                tri = delaunay(PX,PY);
                trisurf(tri,PX,PY,PZ,'LineStyle','none','FaceColor','interp','Marker','.','MarkerSize',size_markers,'DisplayName','Height Plot','FaceLighting','phong');
            catch err
                err
            end

            % Set axis labels
            setLabels(sptitle,v1name,v2name,tname);
        end
    end
end

%% plot2DHeightfieldNG: Plot single function over two variables as 2D Heightfield
function [PX,PY,PZ]=plot2DHeightfieldNG(num_table,num_params,v1id,v2id,tvid,intv1,intv2)

    global name_supp;
    
    % Get Variable IDS
    if ischar(v1id) v1id = find(ismember(num_params, v1id)==1); end
    if ischar(v2id) v2id = find(ismember(num_params, v2id)==1); end
    if ischar(tvid) tvid = find(ismember(num_params, tvid)==1); end
    
    v1name = char(num_params(v1id));
    v2name = char(num_params(v2id));
    tname = char(num_params(tvid));
    
    % Set Figure
    size_fig = [0 0 800 800];
    size_markers = 5.0;
    
    ftitle = ['2D Heightfield: ',name_supp,' ',v1name,',',v2name,' x ',tname];
    setFigure(ftitle,size_fig);
    colormap jet;
    
    % Iterate group data
    %PX = num_table(:,v1id);
    %PY = num_table(:,v2id);
    %PZ = num_table(:,tvid);
    [PX,PY,PZ] = getMeanSurface(num_table,num_params,v1id,v2id,tvid);
    
    % Apply interval filters
    if exist('intv1','var')
         mpx = ~logical((PX>=intv1(1)).*(PX<=intv1(2)));
         PX(mpx)=[]; PY(mpx)=[]; PZ(mpx)=[];
    end
    
    if exist('intv2','var')
         mpy = ~logical((PY>=intv2(1)).*(PY<=intv2(2)));
         PX(mpy)=[]; PY(mpy)=[]; PZ(mpy)=[];
    end

    % Create subplot
    hold on;
    colorbar;

    % Remove undefined values
    PX = checkTable(PX);
    PY = checkTable(PY);
    PZ = checkTable(PZ);

    % Check for remaining values
    if (~isempty(PZ))

        % Try to construct delaunay surface
        try
            
            tri = delaunay(PX,PY);
            trisurf(tri,PX,PY,PZ,'LineStyle','none','FaceColor','interp','Marker','.','MarkerSize',size_markers,'DisplayName','Height Plot','FaceLighting','phong');
            plot3(PX,PY,PZ,'LineStyle','none','Marker','.','MarkerSize',size_markers);
            
            %RPX = PX+rand(size(PX,1),1)*0.5;
            %RPY = PY+rand(size(PX,1),1)*0.5;
            %tri = delaunay(RPX+rand(size(PX,1),1)*0.3,PY+rand(size(PX,1),1)*0.3);
            %trisurf(tri,RPX,RPY,PZ,'FaceColor','interp','Marker','.','MarkerSize',size_markers,'DisplayName','Height Plot','FaceLighting','phong');
        
            %dt = DelaunayTri(PX+rand(size(PX,1),1)*0.1,PY+rand(size(PX,1),1)*0.1);
            %triplot(dt);
            
        catch err
            err
        end

        % Set axis labels
        setLabels(ftitle,v1name,v2name,tname);
    end
end

%% setFigure: Create and Style figure
function [figh]=setFigure(ftitle,sizes)

    global color_groups;
    global create_figs;
    
    % Create Figure
    if create_figs 
        figh = figure('Name',ftitle,'Position',sizes);
    else
        figh = get(gcf);
    end
    
    %set(gca, 'ColorOrder', [0 0 1; 0 1 0; 1 0 0],'LineStyleOrder',{'-*',':','o'},'NextPlot','ReplaceChildren')
    set(gca, 'ColorOrder',color_groups,'NextPlot','ReplaceChildren');
end

%% setLabels: Set axis labels
function setLabels(ftitle,xtitle,ytitle,ztitle)

    global color_label;
    global font;
    
    % Replace special chars
    ftitle = strrep(ftitle,'_',' ');
    xtitle = strrep(xtitle,'_',' ');
    ytitle = strrep(ytitle,'_',' ');
    
    size_data = 10;
    size_values = 8;
    size_axelabel = 12;
    size_title = 12;
    
    axh = get(gcf,'CurrentAxes');
    if exist('xtitle','var') set(get(axh,'XLabel'),'String',xtitle,'FontName',font,'FontSize',size_axelabel,'FontWeight','bold'); end
    if exist('ytitle','var') set(get(axh,'YLabel'),'String',ytitle,'FontName',font,'FontSize',size_axelabel,'FontWeight','bold'); end
    if exist('ztitle','var') set(get(axh,'ZLabel'),'String',ztitle,'FontName',font,'FontSize',size_axelabel,'FontWeight','bold'); end
    
    set(gca,'Box','on');
    set(gca,'XGrid','on');
    set(gca,'YGrid','on');
    set(gca,'FontSize',size_values);
    set(gca,'FontName',font);
    set(gca,'XColor',color_label);
    set(gca,'YColor',color_label);
    set(gca,'ButtonDownFcn',@clickCallback);
    
    title(ftitle,'FontWeight','bold','FontSize',size_title,'FontName',font,'Color',color_label);
end

%% clickCallback: Window click interaction callback
function clickCallback(gcbo,eventdata,handles)
    
    global num_params;
    global num_data;
    global cur_field;
    %get(gcbo)
    
    % Get axes labels
    xpname = strrep(get(get(gcbo,'XLabel'),'String'),' ','_');
    xpid = find(ismember(num_params, xpname)==1);
    ypname = strrep(get(get(gcbo,'YLabel'),'String'),' ','_');
    ypid = find(ismember(num_params, ypname)==1);
    zpname = strrep(get(get(gcbo,'ZLabel'),'String'),' ','_');
    zpid = find(ismember(num_params, zpname)==1);
    
    % Get current position
    curpos = get(gca,'CurrentPoint');
    xval = curpos(1,1);
    yval = curpos(1,2);
    zval = curpos(1,3);
    
    disp([xpname,'(',num2str(xpid),') = ',num2str(xval)]);
    disp([ypname,'(',num2str(ypid),') = ',num2str(yval)]);
    disp([zpname,'(',num2str(zpid),') = ',num2str(zval)]);
    
    % Find closest data entry
    values = num_data(:,[xpid,ypid,zpid]);
    vsize = size(values,2);
    
    if vsize==3
        cpid = dsearchn(values,[xval,yval,zval]);
    elseif vsize==2
        cpid = dsearchn(values,[xval,yval]);
    end
    
    vres = {num_params,num_data(cpid,:)'};
    disp(['Result ',xpname,' = ',num2str(num_data(cpid,xpid))])
    disp(['Result ',ypname,' = ',num2str(num_data(cpid,ypid))])
    disp(['Result ','METHOD',' = ',num2str(num_data(cpid,11))])
    
    % Show preview
    ExtractPreview(cur_field,num_data(cpid,:),num_params);
end

%% getParameterFiles: Get all files in folder
function [fileentries,params,filelist]=getParameterFiles(path,ftype)
    
    % Init parameter struct
    global PARAM_STRUCT;
    global filetype;
    
    PARAM_STRUCT = struct();
    filetype = ftype;
    
    % Get file list
    if(exist(path,'dir'))
        filelist = dir([path,filetype]);
        fileentries = arrayfun(@extractParametersGlobal, filelist, 'UniformOutput', false);
        params = fieldnames(PARAM_STRUCT);
        
        % add path information
        for flc=1:size(filelist,1)
            filelist(flc).path = path;
        end
    else
        str = ['[Path not found. (',path,')]']
    end
end

%% extractParametersGlobal: Evaluate file name
function [evstruct]=extractParametersGlobal(entry)
    
    global PARAM_STRUCT;
    global filetype;

    [evstruct,PARAM_STRUCT]=ExtractParameters(entry.name,filetype,PARAM_STRUCT);
end

%% saveTableData: Save temporary tables
function saveTableData(data,params,num_data,num_params,name_add)

    % Create struct
    tfstruct = struct();
    
    % Create timestamp
    tfstruct.('TIMESTAMP')=clock;
    tfstruct.('DATE')=date;
    
    % write data to struct
    tfstruct.('DATA') = data;
    tfstruct.('PARAMETERS') = params;
    tfstruct.('NUMERIC_DATA') = num_data;
    tfstruct.('NUMERIC_PARAMETERS') = num_params;
    
    % save struct
    fsname = ['TableData_',name_add,'.mat'];
    save(fsname, '-struct', 'tfstruct');
end

%% loadTableData: Load temporary tables
function [data,params,num_data,num_params]=loadTableData(name_add)
    
    % Init variables
	data = [];
    params={};
    num_data = [];
    num_params={};
    
    tic;
    
    % get struct name
    fsname = ['TableData_',name_add,'.mat'];
    
    if exist(fsname,'file')
        
        %load struct
        tfstruct = load(fsname)

        % get data from struct
        data = tfstruct.('DATA');
        params = tfstruct.('PARAMETERS');
        num_data = tfstruct.('NUMERIC_DATA');
        num_params = tfstruct.('NUMERIC_PARAMETERS');
        
        str = ['Loading temporary files successfull.']
    else
        str = ['Temporary file not found! (',fsname,')']
    end
    
    toc;
end

%% checkTable: Remove all NaN and Inf entries
function [table]=checkTable(table)

    % Raplace invalid values
    table(isnan(table)) = 0;
    
    temp = table;
    temp(isinf(table)) = 0;
    tmax = max(max(temp));
    table(isinf(table)) = tmax;
    
end

%% evaluateParameterFiles: Get all files recursively
% function [data,file_num,par_num]=evaluateParameterFiles(basepath,filetype)
%     
%     
%     dl = dir(basepath)
%     dl = dl(3:end);
%     folders = dl([dl.isdir])
%     %arrayfun(getParameterFiles,folders.name);
%     
%     for fc=1:size(folders,1)
%         foldername = [basepath,folders(fc).name,'\']
%         
%         % Get File Entries and Parameter Space
%         [fent,params] = getParameterFiles(foldername,filetype);
%         file_num = size(fent,1)
%         par_num = size(params,1)
%         data = cell(file_num,par_num);
% 
%         if (file_num>0 && par_num>0)
%             % Create data entries
%             for fc=1:file_num
%                 fentry = fent{fc};
% 
%                 for pc=1:par_num
%                     pval = fentry.(params{pc});
%                     data(fc,pc) = {pval};
%                 end
%             end
% 
%             fig = figure('Position',[0 0 1400 900]);
%             table = uitable('Parent',fig,'Data',data,'ColumnName',params','Position',[20 20 1200 800]);
%         end
%     end
% end